/**
 * @file   zaurss.cpp
 * @author NOKUBI Takatsugu
 * @date   Wed Dec 17 13:50:38 2003
 * 
 * @brief  ZauRSS main window class
 * 
 * $Id$
 */

#include "zaurss.h"
#include "zrss.h"

#include <qpe/qcopenvelope_qws.h>

#include <qframe.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qtable.h>
#include <qtextbrowser.h>
#include <qaction.h>
#include <qpetoolbar.h>
#include <qpemenubar.h>
#include <qcstring.h>
#include <qurloperator.h>
#include <qstring.h>
#include <qnetworkprotocol.h>

#include "qinputdialog.h"

extern "C" {
#include <http_fetcher.h> // use HTTP Fetcher
}
#include <stdlib.h>
#include <string.h>

ZauRSS::ZauRSS(QWidget *parent = 0, const char *name = 0, int wFlags = 0)
  : QMainWindow(parent, name, wFlags)
{
  // initialize
  httpmode = 0;
  httpcont = NULL;
  rss = NULL;

  setName("ZauRSS");
  setCaption(tr("ZauRSS"));
  setToolBarsMovable(FALSE);

  QPEToolBar *toolBar = new QPEToolBar(this, "tool");
  toolBar->setHorizontalStretchable(TRUE);
  addToolBar(toolBar, "tool", QMainWindow::Top, TRUE);
  QPEMenuBar *menuBar = new QPEMenuBar(toolBar);
  QPopupMenu *listMenuFile = new QPopupMenu(menuBar);
  menuBar->insertItem(tr("File"), listMenuFile);
  this->setFont(QFont("lcfont", 16, QFont::Normal));
  QAction *actUpdate = new QAction("Update RSS", QString::null, 0, this, 0);
  connect(actUpdate, SIGNAL(activated()), this, SLOT(updaterss()));
  actUpdate->addTo(listMenuFile);
  QAction *actEnterrss = new QAction("Enter RSS", QString::null, 0, this, 0);
  connect(actEnterrss, SIGNAL(activated()), this, SLOT(getrss()));
  actEnterrss->addTo(listMenuFile);
  QAction *actExit = new QAction("Exit", QString::null, 0, this, 0);
  connect(actExit, SIGNAL(activated()), this, SLOT(exitapp()));
  actExit->addTo(listMenuFile);

  topF = new QFrame(this);
  setCentralWidget(topF);
  topL = new QHBoxLayout(topF);
  topL->setSpacing(1);
  topL->setMargin(1);                    

  feeds = new QListView(topF);
  topL->addWidget(feeds);
  rightF = new QFrame(topF);
  topL->addWidget(rightF);
  rightL = new QVBoxLayout(rightF);
  items = new QTable(0, 2, rightF);
  rightL->addWidget(items);
  text = new QTextBrowser(rightF);
  rightL->addWidget(text);

  items->setColumnMovingEnabled(false);
  items->setRowMovingEnabled(false);
  items->setFont(QFont("lcfont", 16, QFont::Normal));
  items->setLeftMargin(0); // hide horizontal header
  items->verticalHeader()->setLabel(0, "");
  items->verticalHeader()->setLabel(1, tr("Title"));
  items->setColumnWidth(0, 32);

  connect(items, SIGNAL(doubleClicked(int, int, int, const QPoint&)),
	  this, SLOT(doubleClicked(int, int, int, const QPoint&)));
}

void ZauRSS::exitapp()
{
  close();
}

void ZauRSS::updaterss()
{
  QString url("http://ukai.org/mu/mu.rdf");
  text->setText("starting update");
  if (readRss(url) == false) {
    text->setText("readRss failure.");
  } else {
      text->setText(QString("updated.\nit has %1 items.")
      	    .arg(rss->getItems().size()));
      updateTable();
  }
}

bool ZauRSS::readRss(QString &url)
{
  if (httpcont != NULL) {
    free(httpcont);
    httpcont = NULL;
  }
  
  int ret = http_fetch(url.latin1(), &httpcont);
  if (ret < 0)
    return false;

  QString str;
  QString str2 = str.fromUtf8(httpcont);
  rss = new ZRss(str2);

  return true;
}

bool ZauRSS::updateTable()
{
    items->setNumRows(rss->getItems().size());
    for (int i = 0; i < rss->getItems().size(); i ++) {
	QTableItem *opt = new QTableItem(items, QTableItem::Never, "o");
	QTableItem *text = new QTableItem(items, QTableItem::Never,
					  rss->getItems().at(i)->title);
	items->setItem(i, 0, opt);
	items->setItem(i, 1, text);
    }
    return true;
}

void ZauRSS::doubleClicked(int row, int, int, const QPoint&)
{
    // send qcop message to open a URL
    QCopEnvelope e("QPE/Application/netfront3", "setDocument(QString)");
    e << rss->getItems().at(row)->link;
    text->setText(QString("open:\n%1").arg(rss->getItems().at(row)->link));
}

void ZauRSS::getrss()
{
    QString url;
    bool ok = FALSE;
    url = QInputDialog::getText(tr("RDF URL"), tr("Enter URL"),
				QString::null, &ok, this);
    if (ok && !url.isEmpty()) {
	text->setText("starting update");
	if (readRss(url) == false) {
	    text->setText("readRss failure.");
	} else {
	    text->setText(QString("updated.\nit has %1 items.")
			  .arg(rss->getItems().size()));
	    updateTable();
	}
    }
}
